#!/bin/bash
 
# squint.cron.sh  -- Quick and DIRTY squid log analysis
#
# Assemble squid logs, and send through squint.pl.  This script gets run on a
# regular basis.  It will install itself in crontab if you give it a chance.
#
# Copyright (c) 2001-2002 Andrew McGill and Leading Edge Business Solutions
# (South Africa).  This software may be redistributed and/or modified only
# under the terms of the GNU General Public Licence, version 2, as published by
# the Free Software Foundation, and contained in the file COPYING. 
# 
# All other rights are reserved, and no warranty is implied.  Despite any
# appearance to the contrary, this software is not fit for ANY purpose you may
# conceive, and should not be used on your system under any circumstances.

BASEDIR="${BASEDIR:=/usr/local/httpd/htdocs/squint}"
SQUINT="${SQUINT:=nice /usr/local/bin/squint.pl}"
REVISION='$Id: squint.cron.sh,v 1.10 2002/11/22 13:59:35 andrewm Exp $'
LOGDIR="${LOGDIR:=/var/squid/logs}"
[ -d $LOGDIR ] || LOGDIR="/var/log/squid/"  
umask 002
# WEEKS=52
 
exec 3>&1 4>&2 >& $BASEDIR/squint.log
date
trap "date" EXIT

# Print a header
function htmlheader()
{
	HEADING="$1"
echo "<HTML>
<HEAD>
<TITLE>$HEADING</TITLE>
</HEAD>
<BODY>
<H1>$HEADING</H1>";
}

function htmlfooter()
{
echo "<HR><FONT size=-4>Generated at `date` by $HOSTNAME<br>$REVISION
</FONT></BODY></HTML>";
}

function checkcrontab()
{
# install CRONTAB if crontab is writable
CRONTAB="/etc/crontab"
[ -w $CRONTAB ] || return

grep -q "squint" $CRONTAB ||
echo "# squint squid reports
00 01 * * Monday  root /usr/local/bin/squint.cron.sh weekly
00 01 * 1 *    root /usr/local/bin/squint.cron.sh monthly
00 01 * * *    root /usr/local/bin/squint.cron.sh daily" >> $CRONTAB && killall -1 cron
}

# Create index.html file
function doupdateindex()
{
mkdir -p $REPORTDIR
# write the index.html file
{
htmlheader "Squint reports";
if [ -f $REPORTDIR/daily/range.txt ] ; then
	echo "<H2>Daily report</H2>"
	echo "<A href=daily/index.html>Daily (`cat $REPORTDIR/daily/range.txt`)</A><BR>";
fi

echo "<H2>Weekly reports</H2>";
for ((NUM=0; NUM<7; NUM++)) ; do
	if [ -f $REPORTDIR/weekly$NUM/range.txt ] ; then
	echo "<A href=weekly$NUM/index.html>`cat $REPORTDIR/weekly$NUM/range.txt`</A><BR>"
	fi
done

echo "<H2>Monthly reports</H2>";
for ((NUM=0; NUM<7; NUM++)) ; do
	if [ -f $REPORTDIR/monthly$NUM/range.txt ] ; then
	echo "<A href=monthly$NUM/index.html>`cat $REPORTDIR/monthly$NUM/range.txt`</A><BR>"
	fi
done

htmlfooter

} > $REPORTDIR/index.html

checkcrontab
} # end of doupdateindex function

# Find the access logs, and concatenate them to stdout in the right order
function cataccesslogs()
{
	find $LOGDIR -maxdepth 1 -name "access.log-*" -mtime -$DAYS | sort |
	while read LOGFILE ; do
		if [ "$LOGFILE" = "${LOGFILE/.gz}" ] ; then
			cat "$LOGFILE"
		else
			zcat "$LOGFILE"
		fi
	done
	[ -r "$LOGDIR/access.log" ] && cat "$LOGDIR/access.log"
}

function doreport()
{
        # doreport DESTINATION FROMTIME-WHEN
        DAYS="$1"
        FROMTIME="$2"
	TOTIME="$3"
	DESTDIR="$REPORTDIR/$4"
	STARTTIME=`date +%s -d "$FROMTIME"`
	ENDTIME=`date +%s -d "$TOTIME"`
	echo "Generating report to $DESTDIR from" \
		"`date +%Y%m%d -d "$FROMTIME"`" "to" \
		"`date +%Y%m%d -d "$TOTIME"`"

	[ -d $DESTDIR ] && rm -r $DESTDIR
	mkdir -p $DESTDIR
	# Unzip files before use ...
	# find $LOGDIR -maxdepth 1 -name "access.log-*.gz" -mtime -$DAYS -exec gunzip {} \;
	cataccesslogs | $SQUINT $DESTDIR $STARTTIME $ENDTIME $USERLIST
	doupdateindex
}

# Rotate the logs in the directory
function rotatereports()
{
	TYPE="$1"	# daily / weekly
	rm -r $REPORTDIR/${TYPE}6
	mv 2>/dev/null -f $REPORTDIR/${TYPE}5 $REPORTDIR/${TYPE}6
	mv 2>/dev/null -f $REPORTDIR/${TYPE}4 $REPORTDIR/${TYPE}5
	mv 2>/dev/null -f $REPORTDIR/${TYPE}3 $REPORTDIR/${TYPE}4
	mv 2>/dev/null -f $REPORTDIR/${TYPE}2 $REPORTDIR/${TYPE}3
	mv 2>/dev/null -f $REPORTDIR/${TYPE}1 $REPORTDIR/${TYPE}2
	mv 2>/dev/null -f $REPORTDIR/${TYPE}0 $REPORTDIR/${TYPE}1
}

function setuphtaccess()
{
HTTPDCONF=/etc/httpd/httpd.conf
[ -w $HTTPDCONF ] && {
grep -q "Added by squint" $HTTPDCONF || echo "
# --squint begin--
# Added by squint - allow .htaccess files in $BASEDIR
<Directory $BASEDIR>
    AllowOverride AuthConfig Limit
</Directory>
# --squint end--
" >> $HTTPDCONF
}

DIRNAME=`cd $REPORTDIR; pwd -P`
AUTHUSERFILE="$DIRNAME/.htpasswd"
touch "$AUTHUSERFILE"
echo "AuthUserFile $AUTHUSERFILE
AuthGroupFile /dev/null
AuthName \"Squint files for `basename $DIRNAME`\"
AuthType Basic

<Limit GET POST>
require valid-user
</Limit>
" > "$REPORTDIR/.htaccess"
echo 1>&2 ".htaccess in $DIRNAME"
}

function doreporting()
{
MONTH="`date '+%Y%m01'`"
case "$1" in
	htaccess)
		setuphtaccess;
		;;
	doupdateindex)
		doupdateindex
		;;
	regen)
		for report in daily weekly0  weekly1  weekly2  weekly3  weekly4 weekly5 weekly6 monthly0 monthly1 monthly2 monthly3 monthly4 monthly5 monthly6 ; do
			doreporting $report 
		done
		;;
	daily)
		# week-so-far report
		doreport 7 "last monday" "now" daily
		;;

	weekly0) doreport 15 "last monday 1 weeks ago" "last monday 0 weeks ago" weekly0 ;;
	weekly1) doreport 21 "last monday 2 weeks ago" "last monday 1 weeks ago" weekly1 ;;
	weekly2) doreport 28 "last monday 3 weeks ago" "last monday 2 weeks ago" weekly2 ;;
	weekly3) doreport 35 "last monday 4 weeks ago" "last monday 3 weeks ago" weekly3 ;;
	weekly4) doreport 42 "last monday 5 weeks ago" "last monday 4 weeks ago" weekly4 ;;
	weekly5) doreport 49 "last monday 6 weeks ago" "last monday 5 weeks ago" weekly5 ;;
	weekly6) doreport 56 "last monday 7 weeks ago" "last monday 6 weeks ago" weekly6 ;;

	weekly) # called on the first day of the week
		rotatereports weekly
		doreport 8 "1 week ago 00:00" "00:00" weekly0
		;;

	monthly0) doreport  62	"$MONTH 1 month ago" "$MONTH 0 month ago" monthly0 ;;
	monthly1) doreport  93	"$MONTH 2 month ago" "$MONTH 1 month ago" monthly1 ;;
	monthly2) doreport 114	"$MONTH 3 month ago" "$MONTH 2 month ago" monthly2 ;;
	monthly3) doreport 144	"$MONTH 4 month ago" "$MONTH 3 month ago" monthly3 ;;
	monthly4) doreport 175	"$MONTH 5 month ago" "$MONTH 4 month ago" monthly4 ;;
	monthly5) doreport 205	"$MONTH 6 month ago" "$MONTH 5 month ago" monthly5 ;;
	monthly6) doreport 236	"$MONTH 7 month ago" "$MONTH 6 month ago" monthly6 ;;

	monthly) # called on the first day of the month
		rotatereports monthly
		doreport 32	"$MONTH 1 month ago" "$MONTH 0 month ago" monthly0 
		;;
	init)
		doupdateindex
		;;
	all)
		doreporting daily
		doreporting weekly
		doreporting monthly
		;;
	*)	
		exec 1>&3 2>&4
		echo "Usage: $0 [daily|weekly|monthly|all]"
		exit 1 # Avoid repeated usage reports
		;;
esac
}

# If there is a dir called list/ in the reports directory, then it is used ...
mkdir -p "$BASEDIR"

LISTDIR=$BASEDIR/lists
if [ -d "$LISTDIR" ] ; then
	for LISTFILE in `ls "$LISTDIR" ` ; do
		[ -f "$LISTDIR/$LISTFILE" ] || continue
		USERLIST="$LISTDIR/$LISTFILE"
		REPORTDIR="$BASEDIR/$LISTFILE"
		doreporting "$@"
	done
fi
# List for all $BASEDIR/all/
USERLIST=""
REPORTDIR="$BASEDIR/all"
doreporting "$@"

{
htmlheader "Squint reports - report list"
echo "<a href=all/index.html>All reports</a><br>"
if [ -d "$LISTDIR" ] ; then
	for LISTFILE in `ls "$LISTDIR" ` ; do
		HREF="$LISTFILE/index.html"
		DESC=$LISTFILE
		[ -f $LISTFILE/desc.txt ] && DESC="`cat $LISTFILE/desc.txt`"
		echo "<a href=$HREF>$DESC</a><br>"
	done
fi
htmlfooter
} > $BASEDIR/index.html
 
exit 0

